@@ -1,25 +1,29 @@
 tbox <-
-function( txt, x, y, w, h,
-          font=2, txt.col="black",
-          lwd=2, border="black", col="transparent", ... )
+function( txt, x, y, wd, ht,
+          font=2, lwd=2,
+          col.txt="black",
+          col.border="black",
+          col.bg="transparent" )
 {
-rect( x-w/2, y-h/2, x+w/2, y+h/2, lwd=lwd, border=border, col=col )
-text( x, y, txt, font=font, col=txt.col, ... )
-invisible( c( x, y, w, h ) )
+rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2, lwd=lwd, border=col.border, col=col.bg )
+text( x, y, txt, font=font, col=col.txt )
+invisible( c( x, y, wd, ht ) )
 }
 
 dbox <-
-function( x, y, w, h=w,
-          font=2, cross.col="black", cwd=5,
-          lwd=2, border="black", col="transparent" )
+function( x, y, wd, ht=wd,
+          font=2, lwd=2, cwd=5,
+          col.cross="black",
+          col.border="black",
+          col.bg="transparent" )
 {
-rect( x-w/2, y-h/2, x+w/2, y+h/2, lwd=lwd, border=border, col=col )
-ch <- h*2/3
+rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2, lwd=lwd, border=col.border, col=col.bg )
+ch <- ht*2/3
 segments( c(x     , x-ch/3),
           c(y+ch/2, y+ch/6),
           c(x     , x+ch/3),
-          c(y-ch/2, y+ch/6), lwd=cwd, col=cross.col )
-invisible( c( x, y, w, h ) )
+          c(y-ch/2, y+ch/6), lwd=cwd, col=col.cross )
+invisible( c( x, y, wd, ht ) )
 }
 
 fillarr <-
@@ -49,14 +53,14 @@ boxarr <-
 function( b1, b2, offset=FALSE, pos=0.6, ... )
 {
 # If we want to offset the arrow a bit to the left
-if( offset ) d <- std.vec( b2[1]-b1[1], b2[2]-b1[2] )
-else d <- c(0,0)
-x1 <- b1[1] - d[2]
-y1 <- b1[2] + d[1]
+d  <- std.vec( b2[1]-b1[1], b2[2]-b1[2] )
+dd <- d * as.logical( offset ) * 4
+x1 <- b1[1] - dd[2]
+y1 <- b1[2] + dd[1]
 w1 <- b1[3]
 h1 <- b1[4]
-x2 <- b2[1] - d[2]
-y2 <- b2[2] + d[1]
+x2 <- b2[1] - dd[2]
+y2 <- b2[2] + dd[1]
 w2 <- b2[3]
 h2 <- b2[4]
 hx1 <- x1 + ifelse( (y2-y1)!=0, (x2-x1)*((h1/2)/abs(y2-y1)), sign(x2-x1)*w1/2 )
@@ -78,7 +82,7 @@ if( abs(vy2-y2) < h2/2 ) { bx2 <- vx2
 fillarr( bx1, by1, bx2, by2, ... )
 invisible( list( x = bx1*(1-pos)+bx2*pos,
                  y = by1*(1-pos)+by2*pos,
-                 d = std.vec(bx2-bx1,by2-by1) ) )
+                 d = d ) )
 }
 
 boxes <- function (obj, ...) UseMethod("boxes")
@@ -97,7 +101,9 @@ function( obj, file,
               scale.D = FALSE,
              digits.D = as.numeric(as.logical(scale.D)),
                 eq.wd = TRUE,
-                eq.ht = TRUE, ... )
+                eq.ht = TRUE,
+               subset = NULL,
+              exclude = NULL, ... )
 {
 if( inherits(obj,"Lexis") ) tm <- tmat( obj )
 else if( is.matrix(obj) & diff(dim(obj))==0 ) tm <- obj
@@ -114,14 +120,30 @@ if( is.null(st.nam) ) st.nam <- paste(1:ncol(tm))
 if( show & inherits( obj, "Lexis" ) )
   {
   SM <- summary(obj,simplify=FALSE,scale=scale.Y)
-  Y <- SM[[1]][-n.st-1,"Risk time:"]
+  Y <- SM[[1]][1:n.st,"Risk time:"]
   D <- SM[[1+as.logical(scale.D)]][1:n.st,1:n.st] * ifelse(scale.D,scale.D,1)
   }
 # No extra line with person-years when they are NA
 if( show.Y ) pl.nam <- gsub("\\\nNA", "",
     paste( st.nam, formatC( Y, format="f", digits=digits.Y, big.mark="," ), sep="\n" ) )
 
-# Her comes the plot
+# Any subsetting:
+if( !is.null(exclude) )
+  {
+  if( is.character(exclude) )
+    exclude <- match( exclude, rownames(tm) )
+  subset <- (1:nrow(tm))[-exclude]
+  }
+if( !is.null(subset) )
+  {
+  if( is.character(exclude) )
+      subset <- match( subset, rownames(tm) )
+  tm <- tm[subset,subset]
+  pl.nam <- pl.nam[subset]
+  n.st <- length(subset)
+  }
+
+# Here comes the plot
 par( mar=c(0,0,0,0), cex=cex )
 plot( NA,
       bty="n",
@@ -135,10 +157,11 @@ if( eq.wd ) wd <- rep( max(wd), length(wd) )
 # If not supplied, ask for positions of boxes
 if( is.list(boxpos) )
   {
-  if( names(boxpos) != c("x","y") )
-    stop( "The list given in 'boxpos=' must have names 'x' and 'y'" )
+  names(boxpos) <- tolower( names(boxpos) )    
+  if( length(intersect(names(boxpos),c("x","y")))<2 )
+    stop( "The list given in 'boxpos=' must have components 'x' and 'y'" )
   if( length(boxpos$x) != n.st | length(boxpos$y) != n.st )
-    stop( "The elements of boxpos must both have length equal to no. states", n.st )
+    stop( "The elements 'x' and 'y' of boxpos must both have length equal to no. states", n.st )
   xx <- boxpos$x
   yy <- boxpos$y
   }
@@ -159,7 +182,7 @@ if( is.logical(boxpos) )
      pt <- locator(1)
      xx[i] <- pt$x
      yy[i] <- pt$y
-     tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i] )
+     tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], ... )
      }
   cat( "\n" )
   }
@@ -179,7 +202,8 @@ for( i in 1:n.st ) for( j in 1:n.st )
     }
   }
 # Redraw the boxes with white background
-for( i in 1:n.st ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], col="white" )
+for( i in 1:n.st ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], col.bg="white" )
+for( i in 1:n.st ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], ... )
 
 #################################################################
 # If we want the code we just print the entire function code,
@@ -189,77 +213,22 @@ if( !missing(file) )
 {
 xx <- round(xx)
 yy <- round(yy)
-
-# Here we start writing the file, beginning with setting the parameters
-cat( '
-obj <- ', deparse( substitute( obj ) ), '\n
-if( inherits(obj,"Lexis") ) tm <- tmat.Lexis( obj ) else tm <- obj
-### Position of the boxes:
-      xx <- c(', paste( xx, collapse=", " ),')
-      yy <- c(', paste( yy, collapse=", " ),')
-     cex <-', cex, '      # How should text and numbers be scaled
-   wmult <-', wmult, '    # Extra box-width relative to string width
-   hmult <-', hmult, '    # Extra box-height relative to string height
-   eq.wd <-', eq.wd, '    # All boxes the same width
-   eq.ht <-', eq.ht, '    # All boxes the same height
-  show.Y <-', show.Y, '   # Show number of person-years in boxes
- scale.Y <-', scale.Y, '  # How should person-years be scaled
-digits.Y <-', digits.Y, ' # How should person-years be printed
-  show.D <-', show.D, '   # Show number of events on arrows
- scale.D <-', scale.D, '  # How should rates be scaled
-digits.D <-', digits.D, ' # How should rates be printed
-                      st.nam <- colnames( tm )
-if( is.null(st.nam) ) st.nam <- paste(1:ncol(tm))
-            pl.nam <- st.nam
-      n.st <- length( st.nam )
-
-# If we want to show person-years and events / rates
-  SM <- summary(obj,simplify=FALSE,scale=scale.Y)
-  Y <- SM[[1]][-n.st-1,"Risk time:"]
-  D <- SM[[1+as.logical(scale.D)]][1:n.st,1:n.st] * ifelse(scale.D,scale.D,1)
-# No extra line with person-years when they are NA
-# and adjust the printing of the person-Years
-if( show.Y )
-  {
-  pl.nam <- gsub( "\\\nNA", "",
-                  paste( st.nam,
-                         formatC( Y, format="f", digits=',digits.Y,',
-                                  big.mark=","),
-                         sep="\n" ) )
-  }
-  
-#################################################################
-# Here comes the plot
-par( mar=c(0,0,0,0), cex=cex )
-plot( NA, bty="n",
-      xlim=0:1*100, ylim=0:1*100, xaxt="n", yaxt="n", xlab="", ylab="" )
-# String height and width only meaningful after a plot has been called
-ht <- strheight( pl.nam ) * hmult
-wd <- strwidth(  pl.nam ) * wmult
-
-# Should all boxes be of same height / width:
-if( eq.ht ) ht <- rep( max(ht), length(ht) )
-if( eq.wd ) wd <- rep( max(wd), length(wd) )
-
-# Plot the boxes
-b <- list()
-for( i in 1:n.st )
-   b[[i]] <- tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i] )
-
-# Plot the arrows and the text along them
-for( i in 1:n.st ) for( j in 1:n.st )
-   {
-   if( !is.na(tm[i,j]) )
-     {
-     arr <- boxarr( b[[i]], b[[j]], offset=!is.na(tm[j,i]) )
-     if( show.D ) text( arr$x-arr$d[2], arr$y+arr$d[1],
-                        formatC( D[i,j], format="f", digits=digits.D ),
-                        adj=as.numeric(c(arr$d[2]>0,arr$d[1]<0)),
-                        font=2, col="black" )
-     }
-   }
-# Redraw the boxes with white background to move the arrows behind them
-for( i in 1:n.st ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], col="white" )
-     ', file=file )
-}
+# Here we write the code-file which will reproduce the plot
+cat('
+boxes.Lexis( ', deparse( substitute( obj ) ),',
+           boxpos = list( x=c(', paste( xx, collapse=", " ),'),
+                          y=c(', paste( yy, collapse=", " ),') ),
+              cex =', cex,',\t # How should text and numbers be scaled
+            wmult =', wmult,'\t # Extra box-width relative to string width
+            hmult =', hmult,',\t # Extra box-height relative to string height
+            eq.wd =', eq.wd,',\t # All boxes the same width
+            eq.ht =', eq.ht,',\t # All boxes the same height
+           show.Y =', show.Y,',\t # Show number of person-years in boxes
+          scale.Y =', scale.Y,',\t\t # How should person-years be scaled
+         digits.Y =', digits.Y,',\t\t # How should person-years be printed
+           show.D =', show.D,',\t # Show number of events on arrows
+          scale.D =', scale.D,',\t # How should rates be scaled
+         digits.D =', digits.D,')\t\t # How should rates be printed \n',
+ file=file )
 }
+}
\ No newline at end of file
